@@ -16,9 +16,13 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import importlib
+import pprint
+import textwrap
+from dataclasses import dataclass
 from types import ModuleType
-from typing import Any, Dict, Iterable, NamedTuple, Optional, Tuple, Type
+from typing import Any, Dict, Iterable, Optional, Tuple, Type, Union
 
+import gitlab
 from gitlab import types as g_types
 from gitlab.exceptions import GitlabParsingError
 
@@ -32,26 +36,43 @@ __all__ = [
 ]
 
 
-class RESTObject(object):
+_URL_ATTRIBUTE_ERROR = (
+    f"https://python-gitlab.readthedocs.io/en/v{gitlab.__version__}/"
+    f"faq.html#attribute-error-list"
+)
+
+
+class RESTObject:
     """Represents an object built from server data.
 
     It holds the attributes know from the server, and the updated attributes in
     another. This allows smart updates, if the object allows it.
 
     You can redefine ``_id_attr`` in child classes to specify which attribute
-    must be used as uniq ID. ``None`` means that the object can be updated
+    must be used as the unique ID. ``None`` means that the object can be updated
     without ID in the url.
+
+    Likewise, you can define a ``_repr_attr`` in subclasses to specify which
+    attribute should be added as a human-readable identifier when called in the
+    object's ``__repr__()`` method.
     """
 
     _id_attr: Optional[str] = "id"
     _attrs: Dict[str, Any]
+    _created_from_list: bool  # Indicates if object was created from a list() action
     _module: ModuleType
     _parent_attrs: Dict[str, Any]
-    _short_print_attr: Optional[str] = None
+    _repr_attr: Optional[str] = None
     _updated_attrs: Dict[str, Any]
     manager: "RESTManager"
 
-    def __init__(self, manager: "RESTManager", attrs: Dict[str, Any]) -> None:
+    def __init__(
+        self,
+        manager: "RESTManager",
+        attrs: Dict[str, Any],
+        *,
+        created_from_list: bool = False,
+    ) -> None:
         if not isinstance(attrs, dict):
             raise GitlabParsingError(
                 "Attempted to initialize RESTObject with a non-dictionary value: "
@@ -64,6 +85,7 @@ class RESTObject(object):
                 "_attrs": attrs,
                 "_updated_attrs": {},
                 "_module": importlib.import_module(self.__module__),
+                "_created_from_list": created_from_list,
             }
         )
         self.__dict__["_parent_attrs"] = self.manager.parent_attrs
@@ -106,8 +128,22 @@ class RESTObject(object):
             except KeyError:
                 try:
                     return self.__dict__["_parent_attrs"][name]
-                except KeyError:
-                    raise AttributeError(name)
+                except KeyError as exc:
+                    message = (
+                        f"{type(self).__name__!r} object has no attribute {name!r}"
+                    )
+                    if self._created_from_list:
+                        message = (
+                            f"{message}\n\n"
+                            + textwrap.fill(
+                                f"{self.__class__!r} was created via a list() call and "
+                                f"only a subset of the data may be present. To ensure "
+                                f"all data is present get the object using a "
+                                f"get(object.id) call. For more details, see:"
+                            )
+                            + f"\n\n{_URL_ATTRIBUTE_ERROR}"
+                        )
+                    raise AttributeError(message) from exc
 
     def __setattr__(self, name: str, value: Any) -> None:
         self.__dict__["_updated_attrs"][name] = value
@@ -115,60 +151,95 @@ class RESTObject(object):
     def __str__(self) -> str:
         data = self._attrs.copy()
         data.update(self._updated_attrs)
-        return "%s => %s" % (type(self), data)
+        return f"{type(self)} => {data}"
+
+    def pformat(self) -> str:
+        data = self._attrs.copy()
+        data.update(self._updated_attrs)
+        return f"{type(self)} => \n{pprint.pformat(data)}"
+
+    def pprint(self) -> None:
+        print(self.pformat())
 
     def __repr__(self) -> str:
-        if self._id_attr:
-            return "<%s %s:%s>" % (
-                self.__class__.__name__,
-                self._id_attr,
-                self.get_id(),
+        name = self.__class__.__name__
+
+        if (self._id_attr and self._repr_attr) and (self._id_attr != self._repr_attr):
+            return (
+                f"<{name} {self._id_attr}:{self.get_id()} "
+                f"{self._repr_attr}:{getattr(self, self._repr_attr)}>"
             )
-        else:
-            return "<%s>" % self.__class__.__name__
+        if self._id_attr:
+            return f"<{name} {self._id_attr}:{self.get_id()}>"
+        if self._repr_attr:
+            return f"<{name} {self._repr_attr}:{getattr(self, self._repr_attr)}>"
+
+        return f"<{name}>"
 
     def __eq__(self, other: object) -> bool:
         if not isinstance(other, RESTObject):
             return NotImplemented
         if self.get_id() and other.get_id():
             return self.get_id() == other.get_id()
-        return super(RESTObject, self) == other
+        return super() == other
 
     def __ne__(self, other: object) -> bool:
         if not isinstance(other, RESTObject):
             return NotImplemented
         if self.get_id() and other.get_id():
             return self.get_id() != other.get_id()
-        return super(RESTObject, self) != other
+        return super() != other
 
     def __dir__(self) -> Iterable[str]:
-        return set(self.attributes).union(super(RESTObject, self).__dir__())
+        return set(self.attributes).union(super().__dir__())
 
     def __hash__(self) -> int:
         if not self.get_id():
-            return super(RESTObject, self).__hash__()
+            return super().__hash__()
         return hash(self.get_id())
 
     def _create_managers(self) -> None:
-        managers = getattr(self, "_managers", None)
-        if managers is None:
-            return
-
-        for attr, cls_name in self._managers:
+        # NOTE(jlvillal): We are creating our managers by looking at the class
+        # annotations. If an attribute is annotated as being a *Manager type
+        # then we create the manager and assign it to the attribute.
+        for attr, annotation in sorted(self.__annotations__.items()):
+            # We ignore creating a manager for the 'manager' attribute as that
+            # is done in the self.__init__() method
+            if attr in ("manager",):
+                continue
+            if not isinstance(annotation, (type, str)):
+                continue
+            if isinstance(annotation, type):
+                cls_name = annotation.__name__
+            else:
+                cls_name = annotation
+            # All *Manager classes are used except for the base "RESTManager" class
+            if cls_name == "RESTManager" or not cls_name.endswith("Manager"):
+                continue
             cls = getattr(self._module, cls_name)
             manager = cls(self.manager.gitlab, parent=self)
+            # Since we have our own __setattr__ method, we can't use setattr()
             self.__dict__[attr] = manager
 
     def _update_attrs(self, new_attrs: Dict[str, Any]) -> None:
         self.__dict__["_updated_attrs"] = {}
         self.__dict__["_attrs"] = new_attrs
 
-    def get_id(self) -> Any:
+    def get_id(self) -> Optional[Union[int, str]]:
         """Returns the id of the resource."""
         if self._id_attr is None or not hasattr(self, self._id_attr):
             return None
         return getattr(self, self._id_attr)
 
+    @property
+    def encoded_id(self) -> Optional[Union[int, str]]:
+        """Ensure that the ID is url-encoded so that it can be safely used in a URL
+        path"""
+        obj_id = self.get_id()
+        if isinstance(obj_id, str):
+            obj_id = gitlab.utils.EncodedId(obj_id)
+        return obj_id
+
     @property
     def attributes(self) -> Dict[str, Any]:
         d = self.__dict__["_updated_attrs"].copy()
@@ -177,13 +248,13 @@ class RESTObject(object):
         return d
 
 
-class RESTObjectList(object):
+class RESTObjectList:
     """Generator object representing a list of RESTObject's.
 
     This generator uses the Gitlab pagination system to fetch new data when
     required.
 
-    Note: you should not instanciate such objects, they are returned by calls
+    Note: you should not instantiate such objects, they are returned by calls
     to RESTManager.list()
 
     Args:
@@ -220,7 +291,7 @@ class RESTObjectList(object):
 
     def next(self) -> RESTObject:
         data = self._list.next()
-        return self._obj_cls(self.manager, data)
+        return self._obj_cls(self.manager, data, created_from_list=True)
 
     @property
     def current_page(self) -> int:
@@ -244,27 +315,28 @@ class RESTObjectList(object):
         return self._list.next_page
 
     @property
-    def per_page(self) -> int:
+    def per_page(self) -> Optional[int]:
         """The number of items per page."""
         return self._list.per_page
 
     @property
-    def total_pages(self) -> int:
+    def total_pages(self) -> Optional[int]:
         """The total number of pages."""
         return self._list.total_pages
 
     @property
-    def total(self) -> int:
+    def total(self) -> Optional[int]:
         """The total number of items."""
         return self._list.total
 
 
-class RequiredOptional(NamedTuple):
-    required: Tuple[str, ...] = tuple()
-    optional: Tuple[str, ...] = tuple()
+@dataclass(frozen=True)
+class RequiredOptional:
+    required: Tuple[str, ...] = ()
+    optional: Tuple[str, ...] = ()
 
 
-class RESTManager(object):
+class RESTManager:
     """Base class for CRUD operations on objects.
 
     Derived class must define ``_path`` and ``_obj_cls``.
@@ -289,8 +361,7 @@ class RESTManager(object):
         """REST manager constructor.
 
         Args:
-            gl (Gitlab): :class:`~gitlab.Gitlab` connection to use to make
-                         requests.
+            gl: :class:`~gitlab.Gitlab` connection to use to make requests.
             parent: REST object to which the manager is attached.
         """
         self.gitlab = gl
@@ -315,7 +386,7 @@ class RESTManager(object):
             for self_attr, parent_attr in self._from_parent_attrs.items()
         }
         self._parent_attrs = data
-        return path % data
+        return path.format(**data)
 
     @property
     def path(self) -> Optional[str]:
