@@ -1,3 +1,7 @@
+from typing import Any, cast, Dict, Union
+
+import requests
+
 from gitlab import cli
 from gitlab import exceptions as exc
 from gitlab.base import RequiredOptional, RESTManager, RESTObject
@@ -25,7 +29,7 @@ class ProjectKey(SaveMixin, ObjectDeleteMixin, RESTObject):
 
 
 class ProjectKeyManager(CRUDMixin, RESTManager):
-    _path = "/projects/%(project_id)s/deploy_keys"
+    _path = "/projects/{project_id}/deploy_keys"
     _obj_cls = ProjectKey
     _from_parent_attrs = {"project_id": "id"}
     _create_attrs = RequiredOptional(required=("title", "key"), optional=("can_push",))
@@ -33,16 +37,24 @@ class ProjectKeyManager(CRUDMixin, RESTManager):
 
     @cli.register_custom_action("ProjectKeyManager", ("key_id",))
     @exc.on_http_error(exc.GitlabProjectDeployKeyError)
-    def enable(self, key_id, **kwargs):
+    def enable(
+        self, key_id: int, **kwargs: Any
+    ) -> Union[Dict[str, Any], requests.Response]:
         """Enable a deploy key for a project.
 
         Args:
-            key_id (int): The ID of the key to enable
+            key_id: The ID of the key to enable
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
             GitlabProjectDeployKeyError: If the key could not be enabled
+
+        Returns:
+            A dict of the result.
         """
-        path = "%s/%s/enable" % (self.path, key_id)
-        self.gitlab.http_post(path, **kwargs)
+        path = f"{self.path}/{key_id}/enable"
+        return self.gitlab.http_post(path, **kwargs)
+
+    def get(self, id: Union[str, int], lazy: bool = False, **kwargs: Any) -> ProjectKey:
+        return cast(ProjectKey, super().get(id=id, lazy=lazy, **kwargs))
