@@ -1,3 +1,5 @@
+from typing import Any, cast, Optional, Union
+
 from gitlab.base import RequiredOptional, RESTManager, RESTObject
 from gitlab.mixins import (
     CreateMixin,
@@ -8,7 +10,6 @@ from gitlab.mixins import (
     UpdateMixin,
 )
 
-
 __all__ = [
     "ProjectPushRules",
     "ProjectPushRulesManager",
@@ -22,7 +23,7 @@ class ProjectPushRules(SaveMixin, ObjectDeleteMixin, RESTObject):
 class ProjectPushRulesManager(
     GetWithoutIdMixin, CreateMixin, UpdateMixin, DeleteMixin, RESTManager
 ):
-    _path = "/projects/%(project_id)s/push_rule"
+    _path = "/projects/{project_id}/push_rule"
     _obj_cls = ProjectPushRules
     _from_parent_attrs = {"project_id": "id"}
     _create_attrs = RequiredOptional(
@@ -49,3 +50,8 @@ class ProjectPushRulesManager(
             "max_file_size",
         ),
     )
+
+    def get(
+        self, id: Optional[Union[int, str]] = None, **kwargs: Any
+    ) -> Optional[ProjectPushRules]:
+        return cast(Optional[ProjectPushRules], super().get(id=id, **kwargs))
